/*
  LMX2330 "Three wire" controller with Arduino
 Dual PLL, YO4HFU Rx/Tx Converter for Eshail Sattelite
  Source:http://www.g8ajn.tv/dlother3.html
  Modified by YO4HFU / 2019
  ******************************************************
  ARDUINO ATTINY85 pins: 
  Switch = port 4, PIN 3
  PLL CLK = port 2, PIN 7 
  PLL DATA = port 1, PIN 6  
  PLL LATCH = port 0, PIN 5
  
  Switch connected between pin 3 and GND. Is used to change LOs frequency for IF=434MHz (switch open) or IF=145MHz (switch close)
  *******************************************************
  CLOCK WIDTH = 0,01 msec (8MHz)
 */
 
 //Registers calculated by Code Loader 4 software; Take care during Binary-Hexa conversion. MSB first.
 
 // Preset_1 for IF 434MHz
  const unsigned long IF_R_1 = 0x160010; //PLL IF 305MHz; REF=10MHz; CP=2.5MHz; P=8; X4; Normal; FoLD IF/RF Locked
  const unsigned long IF_N_1 = 0x1E09;  
  const unsigned long RF_R_1 = 0x160052; //PLL RF 1965.5MHz; REF=10MHz; CP=500KHz; X4; Normal; FoLD IF/RF Locked
  const unsigned long RF_N_1 = 0xF46F;

 // Preset_2 for IF 145MHz
  const unsigned long IF_R_2 = 0x160014; //PLL IF 594MHz; REF=10MHz; CP=2MHz; X4; Normal; FoLD IF/RF Locked
  const unsigned long IF_N_2 = 0x102425; 
  const unsigned long RF_R_2 = 0x160052;  //PLL RF 2254.5MHz; REF=10MHz; CP=500KHz; X4; Normal; FoLD IF/RF Locked
  const unsigned long RF_N_2 = 0x11877;
  
  int bp;
  int SWstate = 0; //variable for reading the Switch
  unsigned long DTA;
  
  const int LE =  0; // Arduino pins
  const int DATA =  1;
  const int CLK =  2;
  const int SWpin = 4;
  


void setup() {       
  pinMode (LE, OUTPUT);  // latch 
  pinMode (CLK, OUTPUT);  // clock
  pinMode (DATA, OUTPUT);  // data
  pinMode (SWpin, INPUT); //switch input
  digitalWrite(SWpin, HIGH); // turn on pullup resistor
  
  digitalWrite(LE,LOW); // clear LE line
  digitalWrite(CLK,LOW); // clear CLK line


//while(true) { // While enable: Start loop and send SPI signals forever. Better to not use due output jitter.
                 // While disable: Power ON, read Switch and send SPI only one time. Need power restart in case of switch changed.
    
 SWstate = digitalRead(SWpin); //Read Switch input
 if (SWstate == HIGH) { 
 
 
 DTA = IF_R_1;    // Send setup parameters_1
 SendWord();
 DTA = IF_N_1;
 SendWord();
 DTA = RF_R_1;
 SendWord();
 DTA = RF_N_1;
 SendWord();
 }  
 
 else {
 
 
 DTA= IF_R_2;    // Send setup parameters_2
 SendWord();
 DTA = IF_N_2;
 SendWord();
 DTA = RF_R_2;
 SendWord();
 DTA = RF_N_2;
 SendWord();
} 
delay (500); //jitter reduced due SPI refresh

//}   // uncomment togheter with "while true"

}



void loop() {  } // loop closed
   
   
  int SendWord() {
  bp = 22;
  do{
  bp = bp - 1;   //loop through bit positions 22 to 0
  digitalWrite(CLK,LOW); 
  digitalWrite(DATA, bitRead(DTA,bp) ); 
  digitalWrite(CLK,HIGH); 
  }while(bp>0); 
  
  digitalWrite(CLK,LOW); 
  digitalWrite(DATA,LOW);

  digitalWrite(LE,HIGH);   // Set LE to latch data
  delayMicroseconds(4); // 
  digitalWrite(LE,LOW);
}
